"""Library for getting distance from optical sensors for the feedback. Requires the nidaq module.
Assumes the following layout:

    (front)
  Ai0 ----- Ai3
     /     \
     |     |
 Ai1 \_ _ _/ Ai2

 Ai0 -> Front lefts
 Ai1 -> Back left
 Ai2 -> Back right
 Ai3 -> Front right

 This library exports only one function: getLR()"""

import nidaq, numpy, math


def get4LR(readProcedure=nidaq.initNIDAQ()):
    """Returns the four distances in that order: r1,l1,l2,r2"""
    return _VoltageToDistance(readProcedure(1)[0][:4])

def getLR(readProcedure=nidaq.initNIDAQ()):
    """Returns the left and right sensor offset, accessing the nidaq through the supplied read procedure (defaults to initNIDAQ())"""

    #Converts the first four voltages to distances; assumes that l1, l2, r1, r2 are
    #respectively connected to AI1 (Analog Input 1), AI2, AI0 and AI3 on the NIDAQ.
    #r1,l1,l2,r2 = get4LR(readProcedure)
    l1,l2,r2,r1 = get4LR(readProcedure)

    return _GetOffset(l1,l2,r1,r2)

def _VoltageToDistance(v):
    """Convert the voltage (V) at the leads of a Sharp GP2D12 IR Sensor to a distance in meters.
    Reliable only when the distance is between 0.01 and 0.08 meters. Works on numpy arrays as well as scalars."""

    return numpy.exp(numpy.log(142.1/v) / 0.88) / 1000.0

def _GetOffset(l1, l2, r1, r2):
    """Return a two-dimensional matrix of offsets calculated from four scalars (or four 1 dimensional vectors of identical size),
    l1, l2, r1,r2 as described in the paper at [put link here]."""

    ANGLE = 50 * math.pi / 180.0 #This is a predefined constant linked to the plastic sensor mount.
    #If a new sensor mount is manufactured, be sure to update this constant (it corresponds to the angle alpha on the paper
    #mentionned in this function's docstring).

    l = l1 * l2 * numpy.sin(ANGLE) / numpy.sqrt(l1**2 + l2**2 - 2*l1*l2*numpy.cos(ANGLE))
    r = r1 * r2 * numpy.sin(ANGLE) / numpy.sqrt(r1**2 + r2**2 - 2*r1*r2*numpy.cos(ANGLE))
    return numpy.array([l,r])
